aboutsummaryrefslogtreecommitdiffstats
path: root/os
diff options
context:
space:
mode:
authorgdisirio <gdisirio@35acf78f-673a-0410-8e92-d51de3d6d3f4>2009-10-20 17:26:27 +0000
committergdisirio <gdisirio@35acf78f-673a-0410-8e92-d51de3d6d3f4>2009-10-20 17:26:27 +0000
commit7d689a5fae4f0950c8cb8e291744465ea5d2ca93 (patch)
treed9b262059d69cbe25cdcf1c9aa4f0e6f2f342337 /os
parent2c41c0d442aa3cea412fba318d4fe0a7cfd276d6 (diff)
downloadChibiOS-7d689a5fae4f0950c8cb8e291744465ea5d2ca93.tar.gz
ChibiOS-7d689a5fae4f0950c8cb8e291744465ea5d2ca93.tar.bz2
ChibiOS-7d689a5fae4f0950c8cb8e291744465ea5d2ca93.zip
git-svn-id: svn://svn.code.sf.net/p/chibios/svn/trunk@1242 35acf78f-673a-0410-8e92-d51de3d6d3f4
Diffstat (limited to 'os')
-rw-r--r--os/kernel/include/heap.h4
-rw-r--r--os/kernel/include/memcore.h1
-rw-r--r--os/kernel/include/mempools.h49
-rw-r--r--os/kernel/src/chheap.c8
-rw-r--r--os/kernel/src/chmempools.c12
5 files changed, 31 insertions, 43 deletions
diff --git a/os/kernel/include/heap.h b/os/kernel/include/heap.h
index 1f07faa91..aa2f7458e 100644
--- a/os/kernel/include/heap.h
+++ b/os/kernel/include/heap.h
@@ -19,8 +19,8 @@
/**
* @file heap.h
- * @brief Heap macros and structures.
- * @addtogroup heap
+ * @brief Heaps macros and structures.
+ * @addtogroup heaps
* @{
*/
diff --git a/os/kernel/include/memcore.h b/os/kernel/include/memcore.h
index c60d3ad9d..7c11e03c3 100644
--- a/os/kernel/include/memcore.h
+++ b/os/kernel/include/memcore.h
@@ -34,6 +34,7 @@ typedef void *align_t;
/**
* @brief Memory get function.
+ *
* @note This type must be assignment compatible with the @p chMemAlloc()
* function.
*/
diff --git a/os/kernel/src/chheap.c b/os/kernel/src/chheap.c
index e7a975f3d..d1dab1580 100644
--- a/os/kernel/src/chheap.c
+++ b/os/kernel/src/chheap.c
@@ -19,8 +19,8 @@
/**
* @file chheap.c
- * @brief Heap code.
- * @addtogroup heap
+ * @brief Heaps code.
+ * @addtogroup heaps
* @{
*/
@@ -47,7 +47,7 @@
static MemoryHeap default_heap;
/**
- * @brief Initializes the allocator subsystem.
+ * @brief Initializes the default heap.
*
* @note Internal use only.
*/
@@ -63,7 +63,7 @@ void heap_init(void) {
}
/**
- * @brief Initializes a memory heap.
+ * @brief Initializes a memory heap from a static memory area.
*
* @param[out] heapp pointer to a memory heap descriptor to be initialized
* @param[in] buf heap buffer base
diff --git a/os/kernel/src/chmempools.c b/os/kernel/src/chmempools.c
index 76ef39b58..97d619520 100644
--- a/os/kernel/src/chmempools.c
+++ b/os/kernel/src/chmempools.c
@@ -33,19 +33,19 @@
* @param[out] mp pointer to a @p MemoryPool structure
* @param[in] size the size of the objects contained in this memory pool,
* the minimum accepted size is the size of a pointer to void.
+ * @param[in] provider memory provider function for the memory pool or
+ * @p NULL if the pool is not allowed to grow automatically
*
* @note The size is internally aligned to be a multiple of the @p align_t
* type size.
*/
-void chPoolInit(MemoryPool *mp, size_t size) {
+void chPoolInit(MemoryPool *mp, size_t size, memgetfunc_t provider) {
chDbgCheck((mp != NULL) && (size >= sizeof(void *)), "chPoolInit");
mp->mp_next = NULL;
mp->mp_object_size = MEM_ALIGN_SIZE(size);
-#if CH_USE_MEMCORE
- mp->mp_usecore = FALSE;
-#endif
+ mp->mp_provider = provider;
}
/**
@@ -63,8 +63,8 @@ void *chPoolAllocI(MemoryPool *mp) {
if ((objp = mp->mp_next) != NULL)
mp->mp_next = mp->mp_next->ph_next;
#if CH_USE_MEMCORE
- else if (mp->mp_usecore)
- objp = chCoreAllocI(mp->mp_object_size);
+ else if (mp->mp_provider != NULL)
+ objp = mp->mp_provider(mp->mp_object_size);
#endif
return objp;
}
diff --git a/os/kernel/include/mempools.h b/os/kernel/include/mempools.h
index a458b8e34..553534dd3 100644
--- a/os/kernel/include/mempools.h
+++ b/os/kernel/include/mempools.h
@@ -40,12 +40,11 @@ struct pool_header {
* @brief Memory pool descriptor.
*/
typedef struct {
- struct pool_header *mp_next; /**< Pointer to the header. */
- size_t mp_object_size; /**< Memory pool objects size. */
-#if CH_USE_MEMCORE
- bool_t mp_usecore; /**< Feed from the memory code
- allocator if empty. */
-#endif
+ struct pool_header *mp_next; /**< @brief Pointer to the header. */
+ size_t mp_object_size; /**< @brief Memory pool objects
+ size. */
+ memgetfunc_t mp_provider; /**< @brief Memory blocks provider for
+ this pool. */
} MemoryPool;
/**
@@ -53,42 +52,30 @@ typedef struct {
* @details This macro should be used when statically initializing a
* memory pool that is part of a bigger structure.
*
* @param name the name of the memory pool variable
* @param size size of the memory pool contained objects
*/
-#if CH_USE_MEMCORE || defined(__DOXYGEN__)
-#define _MEMORYPOOL_DATA(name, size) {NULL, MEM_ALIGN_SIZE(size), FALSE}
-#else
-#define _MEMORYPOOL_DATA(name, size) {NULL, MEM_ALIGN_SIZE(size)}
-#endif
+#define _MEMORYPOOL_DATA(name, size, provider) \
+ {NULL, MEM_ALIGN_SIZE(size), provider}
/**
- * @brief Static memory pool initializer.
+ * @brief Static memory pool initializer in hungry mode.
* @details Statically initialized memory pools require no explicit
* initialization using @p chPoolInit().
*
- * @param name the name of the memory pool variable
- * @param size size of the memory pool contained objects
- */
-#define MEMORYPOOL_DECL(name, size) \
- MemoryPool name = _MEMORYPOOL_DATA(name, size)
-
-#if CH_USE_MEMCORE || defined(__DOXYGEN__)
-/**
- * @brief Enables or disables the hungry mode.
- * @details If enabled, the hungry mode, makes an empty memory pool feed
- * new objects from the core memory manager.
- *
- * @param[in] mp pointer to a @p MemoryPool structure
- * @param[in] mode hungry mode flag
+ * @param[in] name the name of the memory pool variable
+ * @param[in] size size of the memory pool contained objects
+ * @param[in] provider memory provider function for the memory pool or @p NULL
+ * if the pool is not allowed to grow automatically
*/
-#define chPoolSetHungryMode(mp, mode) ((mp)->mp_usecore = (mode))
-#endif
+#define MEMORYPOOL_DECL(name, size, provider) \
+ MemoryPool name = _MEMORYPOOL_DATA(name, size, provider)
#ifdef __cplusplus
extern "C" {
#endif
- void chPoolInit(MemoryPool *mp, size_t size);
+ void chPoolInit(MemoryPool *mp, size_t size, memgetfunc_t provider);
void *chPoolAllocI(MemoryPool *mp);
void *chPoolAlloc(MemoryPool *mp);
void chPoolFreeI(MemoryPool *mp, void *objp);
@@ -53,42 +52,30 @@ typedef struct {
* memory pool that is part of a bigger structure.
*
- * @param name the name of the memory pool variable
- * @param size size of the memory pool contained objects
+ * @param[in] name the name of the memory pool variable
+ * @param[in] size size of the memory pool contained objects
+ * @param[in] provider memory provider function for the memory pool
*/